@@ -25,15 +25,8 @@
 #endif
 
 #include "lbreakout.h"
-#include "file.h"
 #include "config.h"
 #include "theme.h"
-#include "levels.h"
-#include "player.h"
-#include "display.h"
-#include "paddle.h"
-#include "shrapnells.h"
-#include "bricks.h"
 
 extern SDL_Surface *stk_display;
 extern Config config;
@@ -70,6 +63,7 @@ SDL_Surface *paddle_pic = 0;
 SDL_Surface *paddle_shadow = 0;
 SDL_Surface *weapon_pic = 0;
 SDL_Surface *weapon_shadow = 0;
+SDL_Surface *ammo_pic = 0;
 #ifdef AUDIO_ENABLED
 StkSound *wav_expand = 0, *wav_shrink = 0, *wav_frozen = 0;
 #endif
@@ -274,6 +268,9 @@ void theme_load_bkgnds()
             bkgnds[i] = stk_surface_load( SDL_SWSURFACE, theme_path );
         }
     }
+
+    for ( i = 0; i < bkgnd_count; i++ )
+	    SDL_SetColorKey( bkgnds[i], 0, 0x0 );
 }
 
 /*
@@ -439,6 +436,8 @@ void theme_load( char *name )
     weapon_pic = theme_load_surf( "weapon.png" );
     paddle_shadow = create_shadow( paddle_pic, 0, 0, paddle_pic->w, paddle_pic->h );
     weapon_shadow = create_shadow( weapon_pic, 0, 0, weapon_pic->w, weapon_pic->h );
+    ammo_pic = theme_load_surf( "ammo.png" );
+    SDL_SetColorKey( ammo_pic, SDL_SRCCOLORKEY, 0x0 );
     /* paddle sounds */
 #ifdef AUDIO_ENABLED
     wav_expand = theme_load_sound( "expand.wav" );
@@ -514,7 +513,7 @@ void theme_load( char *name )
             BRICK_HEIGHT );
         /* bricks must have been loaded at this point! */
         for ( i = 0; i < MAP_WIDTH - 2; i++ )
-            stk_surface_blit( brick_pic, BRICK_WIDTH, 0,
+            stk_surface_blit( brick_pic, 0, 0,
                 BRICK_WIDTH, BRICK_HEIGHT, 
                 wall_pic, i * BRICK_WIDTH, 0 );
     }
@@ -545,8 +544,8 @@ void theme_load( char *name )
     /* add version to background */
     mfont->align = STK_FONT_ALIGN_RIGHT | STK_FONT_ALIGN_BOTTOM;
     sprintf( aux, "v%s", VERSION );
-    stk_font_write( mfont, mbkgnd, mbkgnd->w - 2, stk_display->h - 2 - mfont->height, STK_OPAQUE, aux );
-    stk_font_write( mfont, mbkgnd, mbkgnd->w - 2, stk_display->h - 2, STK_OPAQUE, "http://www.lgames.sf.net" );
+    stk_font_write( mfont, mbkgnd, mbkgnd->w - 4, stk_display->h - 4 - mfont->height, STK_OPAQUE, aux );
+    stk_font_write( mfont, mbkgnd, mbkgnd->w - 4, stk_display->h - 4, STK_OPAQUE, "http://lgames.sf.net" );
     /* charts */
     /* load resources */
     cfont = theme_load_font_fixed( "f_small_yellow.png", 32, 96, 8 );
@@ -579,6 +578,7 @@ void theme_delete()
 #endif
     stk_surface_free( &paddle_pic );
     stk_surface_free( &weapon_pic );
+    stk_surface_free( &ammo_pic );
     stk_surface_free( &paddle_shadow );
     stk_surface_free( &weapon_shadow );
 #ifdef AUDIO_ENABLED
